#ifdef USEXBEE
; lets try to configure XBEE to run on HSERIAL
XBEEONHSERIAL con 1

;Project Lynxmotion Phoenix
;Description: Phoenix, control file.
;		The control input subroutine for the phoenix software is placed in this file.
;		Can be used with V2.0 and above
;Configuration version: V1.0
;Date: 08-01-2009
;Programmer: Jeroen Janssen (aka Xan)
;Special Thanks to: Kurt (aka Kurte) Assembly
;
;Hardware setup: DIY XBee
;
;NEW IN V1.0
;	- First Release
;
;
;DIY CONTROLS:
;	- Left Stick	(WalkMode) Body Height / Rotate
;					(Translate Mode) Body Height / Rotate body Y	
;					(Rotate Mode) Body Height / Rotate body Y
;					(Single leg Mode) Move Tars Y	
;
;	- Right Stick	(WalkMode) Walk/Strafe
;			 		(Translate Mode) Translate body X/Z
;					(Rotate Mode) Rotate body X/Z
;					(Single leg Mode) Move Tars X/Z
;
; 	- Left Slider	Speed
;	- Right Slider	Leg Lift Height
;
;	- A				Walk Mode
;	- B				Translate Mode
;	- C				Rotate Mode
;	- D				Single Leg Mode
;	- E				Balance Mode on/off
;
;	- 0				Turn on/off the bot
;
;	- 1-8			(Walk mode) Switch gaits
;	- 1-6			(Single leg mode) Switch legs
;
;====================================================================
;[DEBUG]
'DEBUG_OUT con 1
'DEBUG_VERBOSE con 1
;DEBUG_ENTERLEAVE con 1
;--------------------------------------------------------------------
;[CONSTANTS]
WalkMode		con 0
TranslateMode	con 1
RotateMode		con 2
SingleLegMode	con 3
GPPlayerMode	con 4

;--------------------------------------------------------------------

;[DIY XBee Controller Variables]
LastButton 			var Byte(2)		; The state of the buttons in previous packet
iNumButton			var	byte		; which button is pressed. Only used in a few places.
fGPSequenceRun		var	bit			; Was a sequence started?

fPacketChanged		var bit			; Something change in the packet
bPacketPrev			var	byte(XBEEPACKETSIZE)

i					var	byte
b					var	byte		; 
pT					var	pointer		; temporary pointer


; Feedback messages - First byte is the count
_WALKMODE			bytetable	4,  "Walk"
_TANSLATEMODE		bytetable	9, "Translate"
_ROTATEMODE			bytetable	6, "Rotate"
_SINGLELEG			bytetable	10, "Single Leg"
_BALANCEON			bytetable	6, "Bal On"
_BALANCEOFF			bytetable	7, "Bal Off"
_GPSEQMODE			bytetable	8, "Sequence"
_GPSEQSTART			bytetable	5, "Start"
_GPSEQCOMPLETE		bytetable	 9, "Completed"
_GPSEQDISABLED		bytetable	8, "Disabled"
_GPSEQNOTDEF		bytetable	11, "Not defined"

; Feedback - Gait names *** Warning: make sure counts match as I hop through the strings"
_GATENAMES			bytetable	8, 	"Ripple 6",|
								9, 	"Ripple 12",|
								6,	"Quad 9",|
								5, 	"Tri 4",|
								5, 	"Tri 6",|
								5, 	"Tri 8",|
								7, 	"Wave 12",|
								7, 	"Wave 18"
; Other strings...
_EER_SSC1			bytetable	"EER -" ; 5 characters
_EER_SSC2			bytetable	";2",13 ; 3

; Define a sound to send back...
_GPSNDERR			bytetable 100, 5000/25,80, 4000/25] ' play it a little different...

;
; Each of the packets will be defined throughout this file...
;
ControlMode			var byte
abSeqPtr			var byte(2)

; BUGBUG:: trying to updating leg positions depending on height
XZLength1			var sword
FemurAngle1H 		var sword
TibiaAngle1H 		var	sword




;--------------------------------------------------------------------
InitController:
#ifdef DEBUG_ENTERLEAVE
#ifdef HSP_DEBUG
	hserout HSP_DEBUG, ["Enter: Init Controller", 13]
#else
	serout s_out, i9600, ["Enter: Init Controller", 13]
#endif	
#endif	


;high p4
;high p5
;high p6

	gosub InitXBee
   
 	ControlMode = WalkMode
	fGPSequenceRun = 0		; make sure it is init...
#ifdef DEBUG_ENTERLEAVE
#ifdef HSP_DEBUG
	hserout HSP_DEBUG, ["Exit: Init Controller", 13]
#else
	serout s_out, i9600, ["Exit: Init Controller", 13]
#endif	
#endif	
 
return

;==============================================================================
; [ControlAllowInput] - Pass in a flag to turn on or off the interrupt...
;		Serial bit bang outputs don't work well when there are interrupts
;==============================================================================
_fEnableRCInt	var byte
ControlAllowInput[_fEnableRCInt]:
#ifdef cXBEE_RTS	
		gosub XbeeRTSAllowInput[_fEnableRCInt]
#endif
	return



;==============================================================================
; [Control] - This function will try to receive a packet of information
; 		from the remote control over XBee.
;
; the data in a standard packet is arranged in the following byte order:
;	0 - Buttons High
;	1 - Buttons Low
; 	2 - Right Joystick L/R
;	3 - Right Joystick U/D
;	4 - Left Joystick L/R
;	5 - Left Joystick U/D
; 	6 - Right Slider
;	7 - Left Slider
;==============================================================================
	
ControlInput:
#ifdef DEBUG_ENTERLEAVE
#ifdef HSP_DEBUG
	hserout HSP_DEBUG, ["Enter: Control Input", 13]
#else
	serout s_out, i9600, ["Enter: Control Input", 13]
#endif	
#endif	
	; Not sure the best place to test this, but if we ran a sequence and now
	; it says it is not running, let the user know this.
	if fGPSequenceRun and (GPStart = 0) then
		fGPSequenceRun = 0		; clear out the data...
		gosub XBeeOutputString[@_GPSEQCOMPLETE]
		GOSUB XBeeResetPacketTimeout ; sets _lTimerLastPacket
		
	endif

	; First lets try getting a packet from the XBEE
	gosub ReceiveXBeePacket
	; See if we have a valid packet to process
	if fPacketValid  then
		fPacketChanged = 0	; don't need to worry about slop 
		for i = 0 to XBEEPACKETSIZE-1
			if (bPacket(i) <> bPacketPrev(i)) then
				fPacketChanged = 1
				bPacketPrev(i) = bPacket(i)
			endif
		next
		
		if fPacketChanged  then
			if fPacketForced then
				gosub SendXbeeNewDataOnlyPacket[1]
			endif
#ifdef DEBUG_OUT
#ifdef HSP_DEBUG
			hserout HSP_DEBUG, [bin8 bPacket(PKT_BTNHI)\8, bin8 bPacket(PKT_BTNLOW)\8, " "]
			for i = 2 to 7
				hserout HSP_DEBUG, [dec bPacket(i)," "]
			next
			hserout HSP_DEBUG,[13] 
#else
			serout s_out, i9600, [bin8 bPacket(PKT_BTNHI)\8, bin8 bPacket(PKT_BTNLOW)\8, " "]
			for i = 2 to 7
				serout s_out, i9600, [dec bPacket(i)," "]
			next
			serout s_out, i9600,[13] 
#endif			
#endif

		endif

		; OK lets try "0" button for Start. 
		IF bPacket(PKT_BTNLOW).bit0 and (LastButton(0).bit0 = 0) THEN	;Start Button (0 on keypad) test
			IF(HexOn) THEN
				'Turn off
				Sound CSOUND_PIN,[100\5000,80\4500,60\4000]
				BodyPosX = 0
				BodyPosY = 0
				BodyPosZ = 0
				BodyRotX1 = 0
				BodyRotY1 = 0
				BodyRotZ1 = 0
				TravelLengthX = 0
				TravelLengthZ = 0
				TravelRotationY = 0
				
				SSCTime = 600
				GOSUB ServoDriverStart
				GOSUB ServoDriverCommit
				HexOn = 0
#ifdef DEBUG_MAIN				
#ifdef HSP_DEBUG
				hserout HSP_DEBUG, ["Cycles: ", dec cCycleTime, " Sum:", dec lsumCycleTime, 13]
#else
				serout s_out, i9600, ["Cycles: ", dec cCycleTime, " Sum:", dec lsumCycleTime, 13]
#endif				
#endif				
			ELSE
				'Turn on
				Sound CSOUND_PIN,[60\4000,80\4500,100\5000]
				SSCTime = 200
				HexOn = 1	
#ifdef DEBUG_MAIN				
			  	lsumCycleTime = 0 ;;; bugbug
	  			cCycleTime = 0
#endif
			ENDIF
		ENDIF	
		
		IF HexOn THEN
			IF bPacket(PKT_BTNHI).bit2 and (LastButton(1).bit2 = 0) THEN		;A Button Walk Mode
				sound CSOUND_PIN, [50\4000]
				ControlMode = WalkMode
				gosub XBeeOutputString[@_WALKMODE]
			ENDIF

			IF bPacket(PKT_BTNHI).bit3 and (LastButton(1).bit3 = 0) THEN		;B Button Translate Mode
				sound CSOUND_PIN, [50\4000]
				ControlMode = TranslateMode
				gosub XBeeOutputString[@_TANSLATEMODE]
			ENDIF
      
			IF bPacket(PKT_BTNHI).bit4 and (LastButton(1).bit4 = 0) THEN		;C Button Rotate Mode
				sound CSOUND_PIN, [50\4000]
				ControlMode = RotateMode
				gosub XBeeOutputString[@_ROTATEMODE]
			ENDIF
      
			IF bPacket(PKT_BTNHI).bit5 and (LastButton(1).bit5 = 0) THEN		;D Button Single Leg Mode
				sound CSOUND_PIN, [50\4000]
				IF SelectedLeg=255 THEN ; none
					SelectedLeg=cRF
				ELSEIF ControlMode=SingleLegMode ;Double press to turn all legs down
					SelectedLeg=255 ; none
				ENDIF
				ControlMode=SingleLegMode        
				gosub XBeeOutputString[@_SINGLELEG]
			ENDIF

			IF bPacket(PKT_BTNHI).bit6 and (LastButton(1).bit6 = 0) THEN		; E Button Balance Mode on/of
				IF BalanceMode = 0 THEN
					BalanceMode = 1
					sound CSOUND_PIN,[100\4000, 50\8000]
					gosub XBeeOutputString[@_BALANCEON]
				ELSE
					BalanceMode = 0
					sound CSOUND_PIN,[250\3000]
					gosub XBeeOutputString[@_BALANCEOFF]
				ENDIF  
			ENDIF   

			IF bPacket(PKT_BTNHI).bit7 and (LastButton(1).bit7 = 0) THEN		; F Button GP Player Mode Mode on/off
      			if GPEnable THEN ;F Button GP Player Mode Mode on/off -- SSC supports this mode
					gosub XBeeOutputString[@_GPSEQMODE]
					sound CSOUND_PIN, [50\4000]
					
					BodyPosX = 0
					BodyPosZ = 0
					BodyRotX1 = 0
					BodyRotY1 = 0
					BodyRotZ1 = 0
					TravelLengthX = 0
					TravelLengthZ = 0
					TravelRotationY = 0
					
					SelectedLeg=255 ; none
					SLHold=0
			
					ControlMode = GPPlayerMode
				else
					gosub XBeeOutputString[@_GPSEQDISABLED]
					sound CSOUND_PIN, [50\4000]
				endif
			ENDIF

			; Hack there are several places that use the 1-N buttons to select a number as an index
			; so lets convert our bitmap of which key may be pressed to a number...
			; BUGBUG:: There is probably a cleaner way to convert...
			iNumButton = 0		; assume no button
			if ((bPacket(PKT_BTNLOW) & 0xFE) or (bPacket(PKT_BTNHI) & 0x03)) and ((LastButton(0) & 0xFE) = 0) and ((LastButton(1) & 0x03) = 0) then
				b = bPacket(PKT_BTNLOW) & 0xfe
				if b then
					while not b.bit0
						b = b >> 1
						iNumButton = iNumButton + 1
					wend
				else
					b = bPacket(PKT_BTNHI)
					iNumButton = 8		; start off at bit 8
					while not b.bit0
						b = b >> 1
						iNumButton = iNumButton + 1
					wend
				endif
			endif
			; BUGBUG:: we are using all keys now, may want to reserve some...		
			;Switch gait
			; We will do slightly different here than the RC version as we have a bit per button
			IF ControlMode=WalkMode then
				IF iNumButton  and (iNumButton <= 8) THEN	;1-8 Button Gait select	  
					IF ABS(TravelLengthX)<cTravelDeadZone & ABS(TravelLengthZ)<cTravelDeadZone & ABS(TravelRotationY*2)<cTravelDeadZone THEN
					
						;Switch Gait type
						GaitType = iNumButton-1
						Sound CSOUND_PIN,[50\4000]
						GOSUB GaitSelect
						; For the heck do in assembly!
						pT = @_GATENAMES	; Point to the first of the paint names
						mov.l	@PT:16, er0					; pointer to our strings
						mov.b	@GAITTYPE, r1l				; r1l has the gait type
						beq		_CI_GS_ENDLOOP:8			; passed zero in so done
_CI_GS_LOOP:					
						mov.b	@er0+, r2l					; get the count of bytes in this string and increment to next spot
						extu.w	r2
						extu.l	er2							; convert to 32 bits
						add.l	er2, er0					; and increment to the next string
						dec.b	r1l							; decrement our count
						bne		_CI_GS_LOOP:8				; done
_CI_GS_ENDLOOP:
						mov.l	er0, @PT					; save away our generated pointer					
						; And tell the remote the name for the selected gate
#ifdef DEBUG_OUT
#ifdef HSP_DEBUG
						hserout HSP_DEBUG, ["GT - Addrs: ", hex @_GATENAMES, 13]
						hserout HSP_DEBUG, ["GT: ", dec GaitType, " ", hex pT, "=", hex @pt, 13]
#else
						serout s_out, i9600, ["GT - Addrs: ", hex @_GATENAMES, 13]
						serout s_out, i9600, ["GT: ", dec GaitType, " ", hex pT, "=", hex @pt, 13]
#endif
#endif					
					
						gosub XBeeOutputString[pT]
					ENDIF
				elseif iNumButton = 9 
					' We wish to play with the leg positions... to see if I can make them more up and down...
					'The Y from Pep is computed normally something like: 
					'		Y = FemurLength*Cos(90-FemurAngle) 
					'		+ Tibia Length*cos((90-FemurAngle)+(TibiaAngle - 90))
					'Note: the 90s cancel out if we want the foot flat we know the whole Tibialength will go into Y 
					'		so we can compute the needed femur angle
					'	FemurAngle =90 - ACOS(( Y-TibiaLength)/FemurLength)
					'From that we compute TibiaAngle = -90+FemurAngle
					
					GOSUB GetArcCos [((CHexInitY+BodyPosY-cTibiaLength) * c4DEC)/cFemurLength ] ' not sure yet of the appropriate scale here...
					FemurAngle1H = 900 - AngleRad4*180/3141		; Convert to 1/10 degrees
					TibiaAngle1H = FemurAngle1H - 900;
					' now legs compute XZ value for how far the leg will be from the body given the above angles...
					' should be: CoxaLen + sin(90-FemurAngle)*FemurLength
					gosub GetSinCos[900 - FemurAngle1H]
					XZLength1 = cCoxaLength + (sin4*cFemurLength )/c4dec			; get our length from body...
					' now print this out and see if it looks right..
					hserout HSP_DEBUG, ["Y: ", sdec CHexInitY+BodyPosY-cTibiaLength, " FA: ", sdec FemurAngle1H, |
							" TA:", sdec TibiaAngle1H, " XZ:", sdec XZLength1, " LMX", sdec LegPosX(cLM), 13];
					; now lets see what happens when we change the leg positions...
					for LegIndex  = 0 to 5
						gosub GetSinCos[cCoxaAngle1(LegIndex)]
  						LegPosX(LegIndex) = (cos4 * XZLength1)/c4dec	;Set start positions for each leg
  						LegPosZ(LegIndex) = -(sin4 * XZLength1)/c4dec
						hserout HSP_DEBUG, ["(", sdec LegPosX(LegIndex), ",", sdec LegPosZ(LegIndex),") "]
					next
					hserout HSP_DEBUG, [13]
					bPacket(PKT_RJOYUD) = 127-cTravelDeadZone ; Hack see if we can make it walk 1 step to get to new positions...
				ENDIF
			ENDIF

			;Switch single leg
			IF ControlMode=SingleLegMode THEN	  
				IF iNumButton>=1 & iNumButton<=6 THEN
					Sound CSOUND_PIN,[50\4000]
					SelectedLeg = iNumButton-1
					SLHold=0
				ENDIF
			
				IF iNumButton = 9  THEN ;Switch Directcontrol
			  		sound CSOUND_PIN, [50\4000]
			  		SLHold = SLHold^1 	;Toggle SLHold
				ENDIF
			ELSEIF ControlMode=Walkmode
				SelectedLeg=255	; none
				SLHold=0
			ENDIF

			;Body Height - try to limit to doable positions - can not exceed the length of fermur+tibia minus our initial offset)
			BodyPosY = (bPacket(PKT_LJOYUD) / 3) max (cFemurLength+cTibiaLength - CHexInitY) 
			
			;Leg lift height - Right slider has value 0-255 translate to 30-93
			LegLiftHeight = 30 + bPacket(PKT_RSLIDER)/5
			
			;Walk mode			
			IF (ControlMode=WalkMode) THEN 			
				TravelLengthX = -(bPacket(PKT_RJOYLR) - 128)
				TravelLengthZ = -(bPacket(PKT_RJOYUD) - 128)	
				TravelRotationY = -(bPacket(PKT_LJOYLR) - 128)/3

			ENDIF
			
			;Body move	
			IF (ControlMode=TranslateMode) THEN	
				BodyPosX = (bPacket(PKT_RJOYLR)-128)/2
				BodyPosZ = (bPacket(PKT_RJOYUD)-128)/2
				BodyRotY1 = (bPacket(PKT_LJOYLR)-128)*2
			ENDIF		
			
			;Body rotate	
			IF (ControlMode=RotateMode) THEN	
				BodyRotX1 = (bPacket(PKT_RJOYUD)-128)*2
				BodyRotY1 = (bPacket(PKT_LJOYLR)-128)*2
				BodyRotZ1 = -(bPacket(PKT_RJOYLR)-128)*2
			ENDIF
			
			;Single Leg Mode
			IF (ControlMode = SingleLegMode) THEN
				SLLegX = (bPacket(PKT_RJOYLR)-128)
				SLLegZ = -(bPacket(PKT_RJOYUD)-128)
				SLLegY = -(bPacket(PKT_LJOYLR)-128)/5
			ENDIF
			
			IF (ControlMode = GPPlayerMode & iNumButton>=1 & iNumButton<=9 ) THEN	;1-9 Button Play GP Seq
				IF (GPStart = 0) THEN
					GPSEQ = iNumButton-1
				ENDIF
				; Lets verify that there is a sequence number defined for the specified sequence. We can ask the SSC-32
				; for the pointer to the sequence, if 0 or -1 we know it is not defined.
				abSeqPtr = 0,0		; assume no sequence
#ifndef SSC_USE_TASERIAL
				serout cSSC_OUT, cSSC_BAUD, ["EER -", dec GPSEQ*2, ";2",13]
				serin cSSC_IN, cSSC_BAUD, 1000, _TO_GPSP, [str abSeqPtr\2] ;read in 2 byte header
_TO_GPSP:				
#else
				gosub TASerOut[cSSC_OUT, cSSC_BAUD, @_EER_SSC1, 5]		; "EER -"
				gosub SSCOutputNum[GPSEQ*2]								; output the number	
				gosub TASerOut[cSSC_OUT, cSSC_BAUD, @_EER_SSC2, 3]		; ";2",13
				gosub TASerIn[cSSC_IN, 0, cSSC_BAUD, @abSeqPtr, 2, 10000, -1], cGPBytes
#endif				
				IF ((abSeqPtr(0) = 0) and (abSeqPtr(1) = 0)) or ((abSeqPtr(0) = 0xff) and (abSeqPtr(1) = 0xff))	THEN
					gosub XBeeOutputString[@_GPSEQNOTDEF]  ; that sequence was not defined...
					gosub XBeePlaySounds[@_GPSNDERR, 4];	// BUGBUG: maybe should pass number of notes instead?
				ELSE
					; let user know that sequence was started
					gosub XBeeOutputString[@_GPSEQSTART]  ; Tell user sequence started.
#ifdef XBEEONHSERIAL
					gosub WaitHseroutComplete			; the communications interrupts may cause problems.
#endif					
					fGPSequenceRun = 1 ; remember that ran one...
					GPStart = 1
				ENDIF
			ENDIF
			
			;Calculate walking time delay
			InputTimeDelay = 128 - (ABS((bPacket(PKT_RJOYLR)-128)) MIN ABS(((bPacket(PKT_RJOYUD)-128)))) MIN ABS(((bPacket(PKT_LJOYLR)-128))) + (128 -(bPacket(PKT_LSLIDER))/2)


		ENDIF
	  
		LastButton(0) = bPacket(PKT_BTNLOW)
		LastButton(1) = bPacket(PKT_BTNHI)
	else
		; Not a valid packet - we should go to a turned off state as to not walk into things!
		IF(HexOn and (fPacketForced or fPacketEnterSSCMode)) THEN
			'Turn off
			Sound CSOUND_PIN,[100\5000,80\4500,100\5000,60\4000] ' play it a little different...
			BodyPosX = 0
			BodyPosY = 0
			BodyPosZ = 0
			BodyRotX1 = 0
			BodyRotY1 = 0
			BodyRotZ1 = 0
			TravelLengthX = 0
			TravelLengthZ = 0
			TravelRotationY = 0
			
			SSCTime = 600
			GOSUB ServoDriverStart
			GOSUB ServoDriverCommit
			HexOn = 0
		endif
		if fPacketEnterSSCMode then
			' received a packet telling us to enter into SSC-mode.
			' if the hex was on we basically released everything, so now
			' simply cal off to the SSC mode handler
			gosub XBeeHandleSSCMode		' 
		endif
	endif

#ifdef DEBUG_ENTERLEAVE
#ifdef HSP_DEBUG
	hserout HSP_DEBUG, ["Exit: Control Input", 13]
#else
	serout s_out, i9600, ["Exit: Control Input", 13]
#endif	
#endif	

return	
#endif ; usexbee
