#ifdef USERC	
'DEBUG_OUT con 1
DEBUG_LA con 1	; LA -> Logic Analyzer

; Are we using XBEE control
;Project Lynxmotion Phoenix
; Quick and dirt RC version using modified Hitec receiver (KurtE)
; This one is specific to BAP28 as It uses TimerW...
; I never tried an input capture of timerW so this sounds like fun
; will use P12 which is FDIOD...
; Moved to P10 FDIOB as P12 on BB2 has pull-up resistor!.
;====================================================================
CNT_PPM_SIGNALS	con	6
awPPMsInput		var	word(CNT_PPM_SIGNALS)	; Saved PPM values
RCC_RLR			con 0
RCC_RUD			con 1
RCC_LUD			con 2
RCC_LLR			con 3
RCC_KNOB		con 5
RCC_SWITCH		con 4

DEADZONE		con 5

TIMERW_INT_ASM	con 1

fPPMsValid		var	byte					; Is the PPMS stored valid?
fPPMsChanged	var	bit						; have the values changed?
fPPMsValidSinceLastControlCall var bit		; Long name... but hopefully self expanitory...
awPPMs		var	word(CNT_PPM_SIGNALS)	; Saved PPM values
ControlMode			var byte

i				var byte
;fChanged		var bit
_fSwitchOn		var bit;					; is the switch on the receiver on or off???
_fSwitchOnPrev	var	bit						; previous state.
_bErrorCount	var	byte					; allow so many errors before we error out.
MAXRCERRORCNT	con 5
;-----------------------------------------------------------------------
; Variables used by interrupt handler... 
_lLastPulseTime	var	long			; time of the last pulse...	
_lNewPulseTime	var	long				; new pulse time
_wPulseWidth	var	word				; what is the pulse width
_bNextChannel	var	byte				; What is the next channel we are expecting

_wGRB			var	word			; hold on to the GRB
_wGRBPrev		var	word			; last GRB time


;--------------------------------------------------------------------
;[CONSTANTS]
WalkMode		con 0
TranslateMode	con 1
RotateMode		con 2
SingleLegMode	con 3
GPPlayerMode	con 4

;--------------------------------------------------------------------

;--------------------------------------------------------------------
;[InitController] Initialize the PS2 controller
;    Warning: this version has been updated to allow both XBee and
; 			PS2 to control the robot... Which ever one issues a start
;			will take over until it does a stop...
;--------------------------------------------------------------------
#ifdef USEXBEE
InitRCController:
#else
InitController:
#endif
;PS2 controller
	serout s_out, i9600, ["Control Init",13];
	input p10			; make sure p10 starts off as an input pin
	TCRW = 0x30			; count by clock / 8
	TMRW = 0x80			; make sure the counter is running
	TSRW = 0			; clear out all status
	TIOR0 = 0x40		; GRB set to input capture on rising edge
	TCNT = 0			; reset the counter
	TIERW.bit1 = 1		; setup to interrupt on input match
	ONASMINTERRUPT TIMERWINT, HANDLE_TIMERW_ASM
	_bNextChannel = CNT_PPM_SIGNALS+1 ; signal to initialize
	ENABLE TIMERWINT_IMIEB
#ifdef DEBUG_LA	; LA -> Logic Analyzer
	high p4
	pause 50
	low p4
	high p5
	high p6
#endif	
	_bErrorCount = 0
	return

;==============================================================================
; [ControlAllowInput] - Pass in a flag to turn on or off the interrupt...
;		Serial bit bang outputs don't work well when there are interrupts
;==============================================================================
_fEnableRCInt	var byte
ControlAllowInput[_fEnableRCInt]:
;	serout s_out, i9600, ["CAI ", dec _fEnableRCInt, 13];
	if _fEnableRCInt then
		fPPMsValid = 0		; say that they are not valid
		_bNextChannel = CNT_PPM_SIGNALS+1 ; signal to initialize
		TSRW = 0			; clear out all status
#ifdef DEBUG_LA
'		high p8
#endif		
		ENABLE TIMERWINT_IMIEB ; enable the interrupt
	else		
		DISABLE TIMERWINT_IMIEB ; enable the interrupt

		; If the data was valid at this point then save away the information 
		if fPPMsValid then 
			if not fPPMsValidSinceLastControlCall then
				fPPMSChanged = 0  ' first time valid since so update our changed indicator
				fPPMsValidSinceLastControlCall = 1		' remember that we were here!
			endif
			for i = 0 to CNT_PPM_SIGNALS-1		; 
				if  (abs(awPPMsInput(i) - 1500) < DEADZONE) and ((i = RCC_LLR) or (i = RCC_RLR) or (i = RCC_RUD)) then
					if awPPMs(i) <> 1500 then
						awPPMs(i) = 1500
						fPPMsChanged = 1
					endif
				else
					if awPPMsInput(i) <> awPPMs(i) then
						awPPMs(i) = awPPMsInput(i)
						fPPMsChanged = 1;
					endif
				endif
			next
		endif	
#ifdef DEBUG_LA
'		low p8
#endif
	endif
	return

;==============================================================================
; [ControlRCInput] - if both controllers
; [ControlInput] - This function will try to receive a packet of information
; 		from the remote control over XBee.
;
; the data in a standard packet is arranged in the following byte order:
;	0 - Buttons High
;	1 - Buttons Low
; 	2 - Right Joystick L/R
;	3 - Right Joystick U/D
;	4 - Left Joystick L/R
;	5 - Left Joystick U/D
; 	6 - Right Slider
;	7 - Left Slider
;==============================================================================
ControlInput:
	; Since we invalidate the data each time we enable/disable the input, I
	; will not simply rely on the is valid data, but will also check for new
	; valid data when we disable the receiver and use that if it is new since
	; the last time we called ControlInput.  
	; 
	if (not fPPMsValid) and (not fPPMsValidSinceLastControlCall) then
		pause 80	; if nothing was valid before give it another chance
	endif
	if fPPMsValid then 
		; see if anything changed...
		if not fPPMsValidSinceLastControlCall then
			fPPMsChanged = 0	; only say nothing changed if we have not already cached a new value
		endif
		; bugbug I am going to normalize the joystick values that autocenter to have a deadzone...
		for i = 0 to CNT_PPM_SIGNALS-1		; 
			if  (abs(awPPMsInput(i) - 1500) < DEADZONE) and ((i = RCC_LLR) or (i = RCC_RLR) or (i = RCC_RUD)) then
				if awPPMs(i) <> 1500 then
					awPPMs(i) = 1500
					fPPMsChanged = 1
				endif
			else
				if awPPMsInput(i) <> awPPMs(i) then
					awPPMs(i) = awPPMsInput(i)
					fPPMsChanged = 1;
				endif
			endif
		next
	endif	

	if fPPMsValid or fPPMsValidSinceLastControlCall then 
#ifdef DEBUG_OUT
		if fPPMsChanged then	
			gosub ControlAllowInput[0]			; disable the interrupts to allow serout to maybe work...
			for i = 0 to CNT_PPM_SIGNALS-1
				serout s_out, i9600, [dec awPPMs(i)," "]
			next
			serout s_out, i9600,[13] 
			gosub ControlAllowInput[1];			; reenable the interrupts
		endif
#endif
		; See if we just got a valid packet and the robot was not previously on...
		IF(not HexOn) THEN
			'Turn on
			Sound CSOUND_PIN,[60\4000,80\4500,100\5000]
			SSCTime = 200
			HexOn = 1	
			_fSwitchOn = (awPPMs(RCC_SWITCH) > 1500)
		ENDIF	
		

		; figure out how to change modes for now try knob
		; split it into maybe 3 sectors < 1350 Walk, to 1675 Translate and above rotate...
		IF HexOn THEN

			if awPPMs(RCC_KNOB) <  1300 then
				if ControlMode <> WalkMode then
					sound CSOUND_PIN, [50\4000]
					ControlMode = WalkMode
				endif
			elseif awPPMs(RCC_KNOB) < 1500
				if ControlMode <> TranslateMode then
					sound CSOUND_PIN, [50\4000, 50\4000]
					ControlMode = TranslateMode
				ENDIF
			elseif awPPMs(RCC_KNOB) < 1700      
				if ControlMode <> RotateMode then
					sound CSOUND_PIN, [50\4000]
					ControlMode = RotateMode
				endif
			else
				if ControlMode <> SingleLegMode then
					ControlMode=SingleLegMode        
					sound CSOUND_PIN, [50\4000]
					SelectedLeg=cRF
				endif

			ENDIF
      
			; BUGBUG - I have not put anything in here for balance yet...
			; likewise Play a sequence...
			_fSwitchOnPrev = _fSwitchOn
			_fSwitchOn = (awPPMs(RCC_SWITCH) > 1500)

			if _fSwitchOn <> _fSwitchOnPrev then	; the switch toggled.
				IF ControlMode=WalkMode THEN	;1-8 Button Gait select	  
					if GaitType < 7 then
						GaitType = GaitType + 1; 
						Sound CSOUND_PIN,[50\4000]
					else
						GaitType = 0
						Sound CSOUND_PIN,[50\4000, 50\4000]
					endif
					;Switch Gait type
					GOSUB GaitSelect
				elseif Controlmode = SinglelegMode
					if SelectedLeg < 5 then
						Sound CSOUND_PIN,[50\4000]
						SelectedLeg = SelectedLeg + 1
					else
						Sound CSOUND_PIN,[50\4000, 50\4000]
						SelectedLeg = 255
					endif
				ENDIF
			ENDIF

			
			;Body Height
			;BodyPosY = bPacket(PKT_LJOYUD) / 2
			BodyPosy = ((awPPMs(RCC_LUD) min 1100) - 1100) / 6
			
			;Leg lift height - Right slider has value 0-255 translate to 30-93
			;LegLiftHeight = 30 + bPacket(PKT_RSLIDER)/5
			LegLiftHeight = 60				; no control of this yet...
			
			;Walk mode			
			IF (ControlMode=WalkMode) THEN 			
				TravelLengthX = -(awPPMs(RCC_RLR) - 1500) / 3
				TravelLengthZ = -(awPPMs(RCC_RUD) - 1500) / 3	
				TravelRotationY = -(awPPMs(RCC_LLR) - 1500)/10

			ENDIF
			
			;Body move	
			IF (ControlMode=TranslateMode) THEN	
				BodyPosX = (awPPMs(RCC_RLR)-1500)/6
				BodyPosZ = (awPPMs(RCC_RUD)-1500)/6
				BodyRotY1 = (awPPMs(RCC_LLR)-1500)/2
			ENDIF		
			
			;Body rotate	
			IF (ControlMode=RotateMode) THEN	
				BodyRotX1 = (awPPMs(RCC_RUD)-1500)/2
				BodyRotY1 = (awPPMs(RCC_LLR)-1500)/2
				BodyRotZ1 = -(awPPMs(RCC_RLR)-1500)/2
			ENDIF

			;Single Leg Mode
			IF (ControlMode = SingleLegMode) THEN
				SLLegX = (awPPMs(RCC_RLR)-1500) / 3
				SLLegZ = -(awPPMs(RCC_RUD)-1500) / 3
				SLLegY = -(awPPMs(RCC_LLR)-1500)/ 10
			ENDIF
			
			;Calculate walking time delay
			InputTimeDelay = 128 - (ABS((awPPMs(RCC_RLR)-1500)/6) MIN ABS(((awPPMs(RCC_RUD)-1500)/6))) MIN ABS(((awPPMs(RCC_LLR)-1500)/6))
  
		ENDIF
	  
	  	; clear out the status so that we know it is fresh the next time we call
		fPPMsValid = 0
		fPPMsValidSinceLastControlCall = 0			; Assume that we have not had any valid data since the last call to here...
		_bErrorCount = 0							; clear out our error count
	else	; we don't have a valid input. see if bot was active is so make it inactive...
		_bErrorCount = _bErrorCount + 1
		IF (_bErrorCount = MAXRCERRORCNT)and HexOn THEN
			'Turn off
			Sound CSOUND_PIN,[100\5000,80\4500,60\4000]
			BodyPosX = 0
			BodyPosY = 0
			BodyPosZ = 0
			BodyRotX1 = 0
			BodyRotY1 = 0
			BodyRotZ1 = 0
			TravelLengthX = 0
			TravelLengthZ = 0
			TravelRotationY = 0
			
			SSCTime = 600
			GOSUB ServoDriverStart
			GOSUB ServoDriverCommit
			HexOn = 0
		endif
		pause 100;	// temporary pause to see what we need...
	endif
	
#ifdef DEBUG_ENTERLEAVE
	serout s_out, i9600, ["Exit: Control Input", 13]
#endif	

return	
		


return

;------------------------------------------------------------------------------
; HANDLE_TIMERW_IMIEB handler - The idea is to capture our current time
; 		value when the interrupt happens and then calculate a delta from the 
;		previous one.  If > some max we know that we are starting up a new run
;		of timings, else save away in the appropriate index value in our array
;		of timeing with 1 per logical channel.
;------------------------------------------------------------------------------
BEGINASMSUB
HANDLE_TIMERW_ASM:
ASM {
	; First see why we woke up.
	push.l	er0
	push.l	er1						; save away the registers we are using
.ifdef DEBUG_LA
	bnot	#4, @PDR5:8				; DEBUG - togle P4...
.endif
	bld		#1, @TSRW:8				; See if GRB match...
	bcc		_RCHTW_CLEANUP:8		; not ours
	bclr	#1, @TSRW:8				; clear it out.
	mov.w	@GRB:16, e1				; Get the new GRB value
	mov.w	@_WGRBPREV:16, r1		; get the saved value to use
	mov.w	e1, @_WGRBPREV:16		; Save away the new one
	sub.w	r1, e1					; subtract the prev from the new to get delta - I think it handles wrap OK...
	bcc		_RCHTW_NGRB:8			; NO Carry so assume no wrap.
	bld		#7, @TSRW				; see if it shows a wrap happened
	bcc		_RCHTW_NGRB:8			; Did not have overflow so fine
	mov.w	#0xffff, e1				; set high value ...
_RCHTW_NGRB:	
	shlr.w	e1						; divide by 2 to get uS
	xor.w	r0,r0					; zero out r0
	mov.b	@_BNEXTCHANNEL:16, r0l	; Load which channel we are using into r0l
	cmp.b	#CNT_PPM_SIGNALS+1, r0l ; See if this is the first interrupt ...
	beq		_RCHTW_INITMODE:8			; we are in the init phase...
	cmp.w	#799,e1					; see if the value is lower than expected
 	bls		_RCHTW_LESS500:8		; < 800 assume an error again

	cmp.w	#2500, e1				; see if > 2500
	bls		_RCHTW_LT2500:8			; 
	; So exceeds what we think should be a valid signal
	cmp.b	#CNT_PPM_SIGNALS-1, r0l	; 
	bhi		_RCHTW_SARTNEWRUN:8	; 
_RCHTW_LESS500:
	mov.b	r0h, @FPPMSVALID:16		; say we are not valid	r0h should be zero
.ifdef DEBUG_LA
;	bnot	#7, @PDR5:8				; P7...
.endif
_RCHTW_INITMODE:
	mov.b	#CNT_PPM_SIGNALS, r0h	; Hack - reuse code below to set channel value...
_RCHTW_SARTNEWRUN:
.ifdef DEBUG_LA
	bnot	#5, @PDR5:8				; DEBUG - togle P5...
.endif
	mov.b	r0h, @_BNEXTCHANNEL:16	; next one should be channel 0
	bra		_RCHTW_CLEANUP:8		; go to cleanup

_RCHTW_LT2500:
	cmp.b	#CNT_PPM_SIGNALS-1, r0l	; see if we are working on a proper channel
	bhi		_RCHTW_CLEANUP:8	; 
	mov.b	r0l, r1l				; save the value
	extu	r0						; 		
	shll.w	r0						; *2
	mov.w	e1,@(AWPPMSINPUT, er0)	; save away the new pulse width into the array
	inc.b	r1l						; increment our counter
	mov.b	r1l,@_BNEXTCHANNEL:16	; save away the updated count
	cmp.b	#CNT_PPM_SIGNALS, r1l	; see if we got a complete set
	bne		_RCHTW_CLEANUP:8
	mov.b	#1, r0l
	mov.b	r0l, @FPPMSVALID:16		; let caller know we have a complete set
.ifdef DEBUG_LA
	bnot	#6, @PDR5:8				; DEBUG - togle P6...
.endif
_RCHTW_CLEANUP:
	pop.l	er1						; restore the registers we used
	pop.l	er0					
	rte								; return from the exception.
}
ENDASMSUB

#endif	; USERC